@@ -1,7 +1,12 @@
 import numpy
-import scipy.linalg
 from numpy.lib.stride_tricks import as_strided, broadcast_arrays
 
+try:
+    import scipy.linalg
+except:
+    pass
+
+
 def vdot(x,y, z = None):
     """
     vectorized dot
@@ -301,6 +306,18 @@ class RawAlgorithmsMixIn:
         xbar_data = out
         cls._amul(ybar_data, y_data, xbar_data)
         
+    @classmethod
+    def _pb_sqrt(cls, ybar_data, x_data, y_data, out = None):
+        if out == None:
+            raise NotImplementedError('should implement that')
+        
+        xbar_data = out
+        tmp = xbar_data.copy() 
+        cls._div(ybar_data, y_data, tmp)
+        tmp /= 2.
+        xbar_data += tmp
+        return xbar_data
+        
         
     @classmethod
     def _log(cls, x_data, out = None):
@@ -587,8 +604,17 @@ class RawAlgorithmsMixIn:
 
         return out                
         
-
+    @classmethod
+    def _outer_pullback(cls, zbar_data, x_data, y_data, z_data, out = None):
+        if out == None:
+            raise NotImplementedError('should implement that')
         
+        (xbar_data, ybar_data) = out
+        
+        xbar_data += cls._dot(zbar_data, y_data, out = xbar_data.copy())
+        ybar_data += cls._dot(zbar_data, x_data, out = ybar_data.copy())
+
+        return out        
 
     @classmethod
     def _inv(cls, x_data, out = None):
